// This file was generated by ecm_setup_version(): DO NOT EDIT!

#ifndef KPIPEWIRE_VERSION_H
#define KPIPEWIRE_VERSION_H

#define KPIPEWIRE_VERSION_STRING "6.6.5"
#define KPIPEWIRE_VERSION_MAJOR 6
#define KPIPEWIRE_VERSION_MINOR 6
#define KPIPEWIRE_VERSION_PATCH 5
#define KPIPEWIRE_VERSION ((6<<16)|(6<<8)|(5))

#endif
